@@ -704,11 +704,12 @@ var _ = Describe("Podman kube generate", func() {
 				// have anything for protocol under the ports as tcp is the default
 				// for k8s
 				Expect(port.Protocol).To(BeEmpty())
-				if port.HostPort == 4008 {
+				switch port.HostPort {
+				case 4008:
 					foundPort400x++
-				} else if port.HostPort == 5008 {
+				case 5008:
 					foundPort500x++
-				} else {
+				default:
 					foundOtherPort++
 				}
 			}
@@ -849,6 +850,57 @@ var _ = Describe("Podman kube generate", func() {
 		Expect(inspect.OutputToString()).To(ContainSubstring(vol1))
 	})
 
+	It("with subpath volume", func() {
+		// We want to verify that generating Volume's subPath is working properly
+		// https://kubernetes.io/docs/concepts/storage/volumes/#using-subpath
+		// by creating a volume with two directories and mounthing them separately
+		volName := "vol-test1"
+		pvcName := fmt.Sprintf("%s-pvc", volName)
+		podmanTest.PodmanExitCleanly("volume", "create", volName)
+
+		mountPoint := "/mnt"
+
+		etcFile := "etcfile"
+		etcDirPath := filepath.Join(mountPoint, "etc")
+		etcSubPath := filepath.Join("/etc", etcFile)
+		etcMountSubPath := filepath.Join(mountPoint, etcSubPath)
+		populateEtcCmd := fmt.Sprintf("mkdir -p %s; touch %s", etcDirPath, etcMountSubPath)
+
+		varFile := "varfile"
+		varDirPath := filepath.Join(mountPoint, "var")
+		varSubPath := filepath.Join("/var", varFile)
+		varMountSubPath := filepath.Join(mountPoint, varSubPath)
+		populateVarCmd := fmt.Sprintf("mkdir -p %s; touch %s", varDirPath, varMountSubPath)
+
+		cmd := fmt.Sprintf("%s; %s", populateEtcCmd, populateVarCmd)
+		vol := fmt.Sprintf("%s:%s", volName, mountPoint)
+		podmanTest.PodmanExitCleanly("run", "-v", vol, ALPINE, "sh", "-c", cmd)
+
+		etcTargetPath := filepath.Join(mountPoint, etcFile)
+		varTargetPath := filepath.Join(mountPoint, varFile)
+		mountTemplate := "type=volume,source=%s,volume-subpath=%s,target=%s"
+		podmanTest.PodmanExitCleanly("run", "-d", "--pod", "new:test1", "--name", "test-ctr",
+			"--mount", fmt.Sprintf(mountTemplate, volName, etcSubPath, etcTargetPath),
+			"--mount", fmt.Sprintf(mountTemplate, volName, varSubPath, varTargetPath),
+			CITEST_IMAGE, "top")
+
+		kube := podmanTest.PodmanExitCleanly("kube", "generate", "test1")
+
+		pod := new(v1.Pod)
+		err = yaml.Unmarshal(kube.Out.Contents(), pod)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(pod.Spec.Containers[0].VolumeMounts).To(ContainElements(v1.VolumeMount{
+			Name:      pvcName,
+			MountPath: etcTargetPath,
+			SubPath:   etcSubPath,
+		}, v1.VolumeMount{
+			Name:      pvcName,
+			MountPath: varTargetPath,
+			SubPath:   varSubPath,
+		}),
+		)
+	})
+
 	It("when bind-mounting '/' and '/root' at the same time ", func() {
 		// Fixes https://github.com/containers/podman/issues/9764
 
@@ -1541,6 +1593,54 @@ USER test1`
 		Expect(kube).Should(ExitWithError(125, "k8s Deployments can only have restartPolicy set to Always"))
 	})
 
+	It("on pod with --type=job", func() {
+		podName := "test-pod"
+		session := podmanTest.Podman([]string{"pod", "create", podName})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"create", "--pod", podName, CITEST_IMAGE, "top"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+		session = podmanTest.Podman([]string{"create", "--pod", podName, CITEST_IMAGE, "sleep", "100"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		kube := podmanTest.Podman([]string{"kube", "generate", "--type", "job", podName})
+		kube.WaitWithDefaultTimeout()
+		Expect(kube).Should(ExitCleanly())
+
+		dep := new(v1.Job)
+		err := yaml.Unmarshal(kube.Out.Contents(), dep)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(dep.Name).To(Equal(podName + "-job"))
+		Expect(dep.Spec.Template.Name).To(Equal(podName))
+		var intone int32 = 1
+		Expect(dep.Spec.Parallelism).To(Equal(&intone))
+		Expect(dep.Spec.Completions).To(Equal(&intone))
+
+		numContainers := 0
+		for range dep.Spec.Template.Spec.Containers {
+			numContainers++
+		}
+		Expect(numContainers).To(Equal(2))
+	})
+
+	It("on pod with --type=job and --restart=always should fail", func() {
+		podName := "test-pod"
+		session := podmanTest.Podman([]string{"pod", "create", "--restart", "always", podName})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		session = podmanTest.Podman([]string{"create", "--pod", podName, CITEST_IMAGE, "top"})
+		session.WaitWithDefaultTimeout()
+		Expect(session).Should(ExitCleanly())
+
+		kube := podmanTest.Podman([]string{"kube", "generate", "--type", "job", podName})
+		kube.WaitWithDefaultTimeout()
+		Expect(kube).Should(ExitWithError(125, "k8s Jobs can not have restartPolicy set to Always; only Never and OnFailure policies allowed"))
+	})
+
 	It("on pod with invalid name", func() {
 		podName := "test_pod"
 		session := podmanTest.Podman([]string{"pod", "create", podName})
